-- lightrays pixeleffect demo
-- löve2d 0.8.0
-- blog.ioxu.com

require "pixeleffects"
require "util"

globaltime = 0
function love.load()
	cog = file_rgbtoa("img/cog1.png")

	love.mouse.setVisible( visible )

	-- BAD global declarations ahead!
	widthmode, heightmode, fullscreenmode, vsyncmode, fsaamode = love.graphics.getMode( )
	if vsyncmode == true then
		vsyncmodestr = "on"
	else
		vsyncmodestr = "off"
	end
	
	wwidth = widthmode
	wheight = heightmode
	
	bgcolour = {to255( HSVtoRGB( 0.75, 0.3, 0.2, 1 ) ) }

	canvasdisplay = "result"

	-- initialise canvas for the bg and rays canvasses.
	-- the rayscanvas is half resolution
	bgcanvas = love.graphics.newCanvas()
	rayscanvas = love.graphics.newCanvas( wwidth/2, wheight/2 )

	-- pixeleffect parameters
	pe_exposure = 0.3--1
	pe_decay = 0.95
	pe_density = 0.4--1
	pe_weight = 0.29--0.1
	pe_lightPositionOnScreen = {0.5,0.5}
	pe_NUM_SAMPLES = 70--100
end

function love.update(dt)
	globaltime = globaltime + dt
	mousex = love.mouse.getX()
	mousey = love.mouse.getY()
end

function love.draw()
	-- reset and clear
	love.graphics.reset()
	love.graphics.setPixelEffect()
	bgcanvas:clear()
	rayscanvas:clear()

	-- draw to the skycanvas
	love.graphics.setCanvas( bgcanvas )
	
	-- draw sky colour
	love.graphics.setColor( bgcolour )
	love.graphics.rectangle( "fill", 0, 0, wwidth, wheight )
	
	-- draw 'sun' at mouse pos
	love.graphics.setColor(255,255,255,255)
	sinm = math.sin( globaltime * 60 )
	sunr = 20 + sinm * 0.75 -- add some life to the sun's size
	love.graphics.circle( "fill", mousex, mousey, sunr + 0.5, 64)
	love.graphics.circle( "line", mousex, mousey, sunr, 64)

	-- draw cogs
	love.graphics.setColor(lerp3( bgcolour, {0,0,0}, 0.3 ))
	love.graphics.draw( cog, 600, 400, globaltime * 0.1, 0.45, 0.45, 415/2, 415/2 )
	love.graphics.setColor(lerp3( bgcolour, {0,0,0}, 0.6 ))
	love.graphics.draw( cog, 200, 160, globaltime * -0.15, 0.75, 0.75, 415/2, 415/2 )
	love.graphics.setColor(0,0,0,255)
	love.graphics.draw( cog, 400, 300, globaltime * 0.2, 1, 1, 415/2, 415/2 )

	-- this is where the pixeleffect is engaged ..
	-- draw bgcanvas into rayscanvas, using the pixeleffect
	love.graphics.setColor(255,255,255,255)
	
	-- set the rayscanvas to recieve the draw
	love.graphics.setCanvas( rayscanvas )
	
	-- activate the rays pixeleffect
	love.graphics.setPixelEffect(fx.lightrays)

	-- send parameters to the pixeleffect
	-- (these should rather be in the keyboard events)
	fx.lightrays:send( "exposure", pe_exposure  )
	fx.lightrays:send( "decay", pe_decay )
	fx.lightrays:send( "density", pe_density + sinm * 0.02  )
	fx.lightrays:send( "weight", pe_weight  )
	fx.lightrays:send( "lightPositionOnScreen", {mousex/wwidth,1-mousey/wheight}  )
	fx.lightrays:send( "NUM_SAMPLES", pe_NUM_SAMPLES  )

	-- draw the background, this draw command is where the pixeleffect is actually used
	-- the bg is drawn at half size because the rayscanvas is half window-resolution
	-- rayscanvas is finally drawn double sized to compensate
	-- love.graphics.draw( bgcanvas )
	love.graphics.draw( bgcanvas, 0, 0, 0, 0.5, 0.5 )
	-- reset the pixeleffect back to none.
	love.graphics.setPixelEffect()

	-- display according to canvasdisplay mode
	love.graphics.setCanvas()
	love.graphics.setColor(255,255,255,255)
	if canvasdisplay == "result" then
		-- this combines the whole effect
		-- draw the background canvas first ..
		love.graphics.draw( bgcanvas )

		-- then draw the rays canvas (the one with the pixeleffect)
		-- (draw rayscanvas twice for extra additive deluxeness)
		love.graphics.setBlendMode( "additive" )
		love.graphics.draw( rayscanvas, 0, 0, 0, 2, 2 )
		love.graphics.draw( rayscanvas, 0, 0, 0, 2, 2  )
	
	elseif canvasdisplay == "background" then
		love.graphics.draw( bgcanvas )

	elseif canvasdisplay == "rays" then
		love.graphics.draw( rayscanvas, 0, 0, 0, 2, 2  )
	end

	-- draw text
	love.graphics.setBlendMode( "alpha" )
	love.graphics.setColor(0,0,0,64)
	love.graphics.rectangle( "fill", 5, 5, 245, 106 )
	
	love.graphics.setColor(255,255,255,255)
	love.graphics.print( string.format("canvas display (b key) : %s", canvasdisplay), 10, 10 )
	love.graphics.print( string.format("exposure (w,s) : %s", pe_exposure), 10, 22 )
	love.graphics.print( string.format("decay (e,d) : %s", pe_decay), 10, 34 )
	love.graphics.print( string.format("weight (r,f) : %s", pe_weight), 10, 46 )
	love.graphics.print( string.format("density (t,g) : %s", pe_density), 10, 58 )
	love.graphics.print( string.format("samples (q,a) : %s", pe_NUM_SAMPLES), 10, 70 )
	love.graphics.print( string.format("vsync (v) : %s", vsyncmodestr ), 10, 82 )
	love.graphics.print( string.format("fps : %s", love.timer.getFPS( ) ), 10, 94 )
end

-- keys
function love.keypressed( )
	
	-- quit on Escape key
	if love.keyboard.isDown("escape") then
		love.event.push('quit')
	end
	
	-- cycle canvas display
	if love.keyboard.isDown("b") then
		if canvasdisplay == "result" then
			canvasdisplay = "background"
		elseif canvasdisplay == "background" then
			canvasdisplay = "rays"
		elseif canvasdisplay == "rays" then
			canvasdisplay = "result"
		end		
	end

	-- vsync
	if love.keyboard.isDown("v") then
		widthmode, heightmode, fullscreenmode, vsyncmode, fsaamode = love.graphics.getMode( )
		if vsyncmode == true then
			love.graphics.setMode( widthmode, heightmode, fullscreenmode, false, fsaamode )
			vsyncmodestr = "off"
		else
			love.graphics.setMode( widthmode, heightmode, fullscreenmode, true, fsaamode )
			vsyncmodestr = "on"
		end
	end

	-- change pixeleffect parameters
	-- exposure
	if love.keyboard.isDown("w") then
		pe_exposure = pe_exposure + 0.1
	elseif love.keyboard.isDown("s") then
		pe_exposure = pe_exposure - 0.1
		if pe_exposure < 0.001 then pe_exposure = 0 end
	-- decay
	elseif love.keyboard.isDown("e") then
		pe_decay = pe_decay + 0.01
	elseif love.keyboard.isDown("d") then
		pe_decay = pe_decay - 0.01
		if pe_decay < 0.001 then pe_decay = 0 end
	-- weight
	elseif love.keyboard.isDown("r") then
		pe_weight = pe_weight + 0.01
	elseif love.keyboard.isDown("f") then
		pe_weight = pe_weight - 0.01
		if pe_weight < 0.001 then pe_weight = 0 end
	-- density
	elseif love.keyboard.isDown("t") then
		pe_density = pe_density + 0.1
	elseif love.keyboard.isDown("g") then
		pe_density = pe_density - 0.1
		if pe_density < 0.001 then pe_density = 0 end
	-- samples
	elseif love.keyboard.isDown("q") then
		pe_NUM_SAMPLES = pe_NUM_SAMPLES + 10
	elseif love.keyboard.isDown("a") then
		pe_NUM_SAMPLES = pe_NUM_SAMPLES - 10
		if pe_NUM_SAMPLES < 0.001 then pe_NUM_SAMPLES = 0 end
	end
end
